@@ -8,7 +8,7 @@ the expected schema structure without requiring external JSON schema libraries.
 
 import sys
 from pathlib import Path
-from typing import Any, Union
+from typing import Any
 
 import yaml  # type: ignore
 from simple_logger.logger import get_logger
@@ -67,7 +67,13 @@ class ConfigValidator:
                 self.errors.append(f"Field '{field}' must be an integer")
 
         # Boolean fields
-        boolean_fields = ["verify-github-ips", "verify-cloudflare-ips", "disable-ssl-warnings"]
+        boolean_fields = [
+            "verify-github-ips",
+            "verify-cloudflare-ips",
+            "disable-ssl-warnings",
+            "mask-sensitive-data",
+            "auto-verify-cherry-picked-prs",
+        ]
         for field in boolean_fields:
             if field in config and not isinstance(config[field], bool):
                 self.errors.append(f"Field '{field}' must be a boolean")
@@ -160,7 +166,7 @@ class ConfigValidator:
                 self.errors.append(f"Repository '{repo_name}' field '{field}' must be a string")
 
         # Boolean fields
-        boolean_fields = ["verified-job", "pre-commit"]
+        boolean_fields = ["verified-job", "pre-commit", "mask-sensitive-data", "auto-verify-cherry-picked-prs"]
         for field in boolean_fields:
             if field in repo_config and not isinstance(repo_config[field], bool):
                 self.errors.append(f"Repository '{repo_name}' field '{field}' must be a boolean")
@@ -233,7 +239,7 @@ class ConfigValidator:
                 self.errors.append(f"Repository '{repo_name}' tox branch '{branch}' must be a string or array")
 
 
-def validate_config_file(config_path: Union[str, Path]) -> bool:
+def validate_config_file(config_path: str | Path) -> bool:
     """
     Validate a configuration file.
 
@@ -244,7 +250,7 @@ def validate_config_file(config_path: Union[str, Path]) -> bool:
         True if valid, False otherwise
     """
     try:
-        with open(config_path, "r") as file_handle:
+        with open(config_path) as file_handle:
             config_data = yaml.safe_load(file_handle)
     except Exception as exception:
         logger = get_logger(name="test_schema_validator")
