@@ -42,7 +42,7 @@ class TestProcessGithubWebhook:
         """Test successful webhook creation when no existing hooks exist."""
         mock_get_repo_api.return_value = mock_repo
 
-        success, message, log_func = process_github_webhook(
+        success, message, _ = process_github_webhook(
             repository_name="test-repo", data=sample_data, webhook_ip="http://example.com", apis_dict=apis_dict
         )
 
@@ -54,7 +54,7 @@ class TestProcessGithubWebhook:
         # Verify webhook creation was called
         mock_repo.create_hook.assert_called_once_with(
             name="web",
-            config={"url": "http://example.com/webhook_server", "content_type": "json"},
+            config={"url": "http://example.com", "content_type": "json"},
             events=["push", "pull_request"],
             active=True,
         )
@@ -70,7 +70,7 @@ class TestProcessGithubWebhook:
         """Test successful webhook creation with secret."""
         mock_get_repo_api.return_value = mock_repo
 
-        success, message, log_func = process_github_webhook(
+        success, _, _ = process_github_webhook(
             repository_name="test-repo",
             data=sample_data,
             webhook_ip="http://example.com",
@@ -84,7 +84,7 @@ class TestProcessGithubWebhook:
         mock_repo.create_hook.assert_called_once_with(
             name="web",
             config={
-                "url": "http://example.com/webhook_server",
+                "url": "http://example.com",
                 "content_type": "json",
                 "secret": "test-secret",  # pragma: allowlist secret
             },
@@ -100,7 +100,7 @@ class TestProcessGithubWebhook:
         mock_get_repo_api.return_value = mock_repo
         data_without_events = {"name": "owner/test-repo"}
 
-        success, message, log_func = process_github_webhook(
+        success, _, _ = process_github_webhook(
             repository_name="test-repo", data=data_without_events, webhook_ip="http://example.com", apis_dict=apis_dict
         )
 
@@ -109,7 +109,7 @@ class TestProcessGithubWebhook:
         # Verify default events are used
         mock_repo.create_hook.assert_called_once_with(
             name="web",
-            config={"url": "http://example.com/webhook_server", "content_type": "json"},
+            config={"url": "http://example.com", "content_type": "json"},
             events=["*"],  # Default events
             active=True,
         )
@@ -125,11 +125,11 @@ class TestProcessGithubWebhook:
         """Test when webhook already exists with same configuration."""
         # Mock existing hook with matching URL
         existing_hook = Mock()
-        existing_hook.config = {"url": "http://example.com/webhook_server", "content_type": "json"}
+        existing_hook.config = {"url": "http://example.com", "content_type": "json"}
         mock_repo.get_hooks.return_value = [existing_hook]
         mock_get_repo_api.return_value = mock_repo
 
-        success, message, log_func = process_github_webhook(
+        success, message, _ = process_github_webhook(
             repository_name="test-repo", data=sample_data, webhook_ip="http://example.com", apis_dict=apis_dict
         )
 
@@ -153,14 +153,14 @@ class TestProcessGithubWebhook:
         # Mock existing hook without secret, but we're adding a secret
         existing_hook = Mock()
         existing_hook.config = {
-            "url": "http://example.com/webhook_server",
+            "url": "http://example.com",
             "content_type": "json",
             # No secret in existing hook
         }
         mock_repo.get_hooks.return_value = [existing_hook]
         mock_get_repo_api.return_value = mock_repo
 
-        success, message, log_func = process_github_webhook(
+        success, _, _ = process_github_webhook(
             repository_name="test-repo",
             data=sample_data,
             webhook_ip="http://example.com",
@@ -177,7 +177,7 @@ class TestProcessGithubWebhook:
         mock_repo.create_hook.assert_called_once_with(
             name="web",
             config={
-                "url": "http://example.com/webhook_server",
+                "url": "http://example.com",
                 "content_type": "json",
                 "secret": "new-secret",  # pragma: allowlist secret
             },
@@ -197,14 +197,14 @@ class TestProcessGithubWebhook:
         # Mock existing hook with secret, but we're removing it
         existing_hook = Mock()
         existing_hook.config = {
-            "url": "http://example.com/webhook_server",
+            "url": "http://example.com",
             "content_type": "json",
             "secret": "old-secret",  # pragma: allowlist secret
         }
         mock_repo.get_hooks.return_value = [existing_hook]
         mock_get_repo_api.return_value = mock_repo
 
-        success, message, log_func = process_github_webhook(
+        success, _, _ = process_github_webhook(
             repository_name="test-repo",
             data=sample_data,
             webhook_ip="http://example.com",
@@ -220,7 +220,7 @@ class TestProcessGithubWebhook:
         # Verify new hook was created without secret
         mock_repo.create_hook.assert_called_once_with(
             name="web",
-            config={"url": "http://example.com/webhook_server", "content_type": "json"},
+            config={"url": "http://example.com", "content_type": "json"},
             events=["push", "pull_request"],
             active=True,
         )
@@ -234,7 +234,7 @@ class TestProcessGithubWebhook:
             }
         }
 
-        success, message, log_func = process_github_webhook(
+        success, message, _ = process_github_webhook(
             repository_name="test-repo", data=sample_data, webhook_ip="http://example.com", apis_dict=apis_dict_no_api
         )
 
@@ -249,7 +249,7 @@ class TestProcessGithubWebhook:
         """Test error handling when repository cannot be found."""
         mock_get_repo_api.return_value = None
 
-        success, message, log_func = process_github_webhook(
+        success, message, _ = process_github_webhook(
             repository_name="test-repo", data=sample_data, webhook_ip="http://example.com", apis_dict=apis_dict
         )
 
@@ -270,7 +270,7 @@ class TestProcessGithubWebhook:
         mock_repo.get_hooks.side_effect = Exception("Permission denied")
         mock_get_repo_api.return_value = mock_repo
 
-        success, message, log_func = process_github_webhook(
+        success, message, _ = process_github_webhook(
             repository_name="test-repo", data=sample_data, webhook_ip="http://example.com", apis_dict=apis_dict
         )
 
@@ -291,15 +291,15 @@ class TestProcessGithubWebhook:
         """Test handling of multiple existing hooks, only matching ones are affected."""
         # Mock multiple existing hooks
         matching_hook = Mock()
-        matching_hook.config = {"url": "http://example.com/webhook_server", "content_type": "json"}
+        matching_hook.config = {"url": "http://example.com", "content_type": "json"}
 
         non_matching_hook = Mock()
-        non_matching_hook.config = {"url": "http://different.com/webhook_server", "content_type": "json"}
+        non_matching_hook.config = {"url": "http://different.com", "content_type": "json"}
 
         mock_repo.get_hooks.return_value = [matching_hook, non_matching_hook]
         mock_get_repo_api.return_value = mock_repo
 
-        success, message, log_func = process_github_webhook(
+        success, message, _ = process_github_webhook(
             repository_name="test-repo", data=sample_data, webhook_ip="http://example.com", apis_dict=apis_dict
         )
 
